@@ -1,6 +1,4 @@
 /*
- * "$Id: plist.c,v 1.18 2008/07/04 14:29:28 rlk Exp $"
- *
  *   Print plug-in for the GIMP.
  *
  *   Copyright 1997-2000 Michael Sweet (mike@easysw.com) and
@@ -17,8 +15,7 @@
  *   for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,8 +31,10 @@
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 #include <locale.h>
+#include <stdlib.h>
 
 #include <sys/types.h>
 #include <signal.h>
@@ -190,7 +189,7 @@ stpui_build_standard_print_command(const stpui_plist_t *plist,
     raw = 0;
   else
     raw = 1;
-  
+
   if (copy_count > 1)
     stp_asprintf(&count_string, "%s %d ",
 		 global_printing_system->copy_count_command, copy_count);
@@ -350,12 +349,6 @@ writefunc(void *file, const char *buf, size_t bytes)
   fwrite(buf, 1, bytes, prn);
 }
 
-static void
-stpui_errfunc(void *file, const char *buf, size_t bytes)
-{
-  g_message("%s",buf);
-}
-
 void
 stpui_printer_initialize(stpui_plist_t *printer)
 {
@@ -376,6 +369,7 @@ stpui_printer_initialize(stpui_plist_t *printer)
   stp_set_errdata(printer->v, stderr);
   stpui_plist_set_copy_count(printer, 1);
   stp_set_string_parameter(printer->v, "InputImageType", image_type);
+  stp_set_string_parameter(printer->v, "JobMode", "Page");
   if (image_raw_channels)
     {
       (void) sprintf(tmp, "%d", image_raw_channels);
@@ -747,7 +741,7 @@ stpui_printrc_load_v0(FILE *fp)
 	}
       else
 	continue;
-      
+
       if (!get_mandatory_string_param(key.v, "Resolution", &lineptr))
 	continue;
       if (!get_mandatory_string_param(key.v, "PageSize", &lineptr))
@@ -918,6 +912,9 @@ stpui_printrc_load_v1(FILE *fp)
 	    case STP_PARAMETER_TYPE_DOUBLE:
 	      stp_set_float_parameter(key.v, keyword, atof(value));
 	      break;
+	    case STP_PARAMETER_TYPE_DIMENSION:
+	      stp_set_dimension_parameter(key.v, keyword, atof(value));
+	      break;
 	    case STP_PARAMETER_TYPE_INT:
 	      stp_set_int_parameter(key.v, keyword, atoi(value));
 	      break;
@@ -957,7 +954,7 @@ stpui_printrc_load_v1(FILE *fp)
 	if (strcmp(current_printer, stpui_plist[i].name) == 0)
 	  stpui_plist_current = i;
     }
-}  
+}
 
 char *stpui_printrc_current_printer = NULL;
 extern FILE *yyin;
@@ -966,7 +963,6 @@ extern int yyparse(void);
 static void
 stpui_printrc_load_v2(FILE *fp)
 {
-  int retval;
   char *locale;
   yyin = fp;
 
@@ -975,7 +971,7 @@ stpui_printrc_load_v2(FILE *fp)
   locale = g_strdup(setlocale(LC_NUMERIC, NULL));
   setlocale(LC_NUMERIC, "C");
 #endif
-  retval = yyparse();
+  (void) yyparse();
 #ifdef HAVE_LOCALE_H
   setlocale(LC_NUMERIC, locale);
   SAFE_FREE(locale);
@@ -1051,6 +1047,7 @@ stpui_printrc_load(void)
 	case 2:
 	case 3:
 	case 4:
+	case 5:
 	  stpui_printrc_load_v2(fp);
 	  break;
 	}
@@ -1088,7 +1085,7 @@ stpui_printrc_save(void)
       fprintf(stderr, "Number of printers: %d\n", stpui_plist_count);
 #endif
 
-      fputs("#PRINTRCv4 written by Gutenprint " PLUG_IN_VERSION "\n\n", fp);
+      fputs("#PRINTRCv5 written by Gutenprint " PLUG_IN_VERSION "\n\n", fp);
 
       fprintf(fp, "Global-Settings:\n");
       fprintf(fp, "  Current-Printer: \"%s\"\n",
@@ -1115,15 +1112,15 @@ stpui_printrc_save(void)
 	  fprintf(fp, "  Output-Filename: \"%s\"\n", p->output_filename);
 	  fprintf(fp, "  Extra-Printer-Options: \"%s\"\n", p->extra_printer_options);
 	  fprintf(fp, "  Custom-Command: \"%s\"\n", p->custom_command);
-	  fprintf(fp, "  Scaling: %.3f\n", p->scaling);
+	  fprintf(fp, "  Scaling: %.6f\n", p->scaling);
 	  fprintf(fp, "  Orientation: %d\n", p->orientation);
 	  fprintf(fp, "  Autosize-Roll-Paper: %d\n", p->auto_size_roll_feed_paper);
 	  fprintf(fp, "  Unit: %d\n", p->unit);
 
-	  fprintf(fp, "  Left: %d\n", stp_get_left(p->v));
-	  fprintf(fp, "  Top: %d\n", stp_get_top(p->v));
-	  fprintf(fp, "  Custom_Page_Width: %d\n", stp_get_page_width(p->v));
-	  fprintf(fp, "  Custom_Page_Height: %d\n", stp_get_page_height(p->v));
+	  fprintf(fp, "  Left: %f\n", stp_get_left(p->v));
+	  fprintf(fp, "  Top: %f\n", stp_get_top(p->v));
+	  fprintf(fp, "  Custom_Page_Width: %f\n", stp_get_page_width(p->v));
+	  fprintf(fp, "  Custom_Page_Height: %f\n", stp_get_page_height(p->v));
 	  fprintf(fp, "  Parameter %s Int True %d\n", copy_count_name,
 		  stpui_plist_get_copy_count(p));
 
@@ -1165,7 +1162,7 @@ stpui_printrc_save(void)
 		case STP_PARAMETER_TYPE_DIMENSION:
 		  if (stp_check_dimension_parameter(p->v, param->name,
 						    STP_PARAMETER_INACTIVE))
-		    fprintf(fp, "  Parameter %s Dimension %s %d\n", param->name,
+		    fprintf(fp, "  Parameter %s Dimension %s %f\n", param->name,
 			    ((stp_get_dimension_parameter_active
 			      (p->v, param->name) == STP_PARAMETER_ACTIVE) ?
 			     "True" : "False"),
@@ -1249,8 +1246,8 @@ stpui_get_system_printers(void)
   char  line[1025];		/* Line from status command */
 
   stpui_system_print_queues = stp_string_list_create();
-  stp_string_list_add_string(stpui_system_print_queues, "",
-			     _("(Default Printer)"));
+  stp_string_list_add_string_unsafe(stpui_system_print_queues, "",
+				    _("(Default Printer)"));
 
  /*
   * Run the command, if any, to get the available printers...
@@ -1281,8 +1278,8 @@ stpui_get_system_printers(void)
 	  if (strlen(line) > 0)
 	    {
 	      if (!stp_string_list_is_present(stpui_system_print_queues, line))
-		stp_string_list_add_string(stpui_system_print_queues,
-					   line, line);
+		stp_string_list_add_string_unsafe(stpui_system_print_queues,
+						  line, line);
 	    }
 	}
       pclose(pfile);
@@ -1380,7 +1377,7 @@ usr1_handler (int sig)
  * x,y refers to file descriptor y duplicated onto file descriptor x.
  * So "<0,3" means input file descriptor 3 (pipefd[0]) dup2'ed onto
  * file descriptor 0.
- * 
+ *
  * fd0 = fd 0
  * fd1 = fd 1
  * fd2 = fd 2
@@ -1391,9 +1388,9 @@ usr1_handler (int sig)
  * fd7 = errfd[0]
  * fd8 = errfd[1]
  *
- * 
+ *
  *                            NORMAL CASE
- * 
+ *
  * PARENT             CHILD 1              CHILD 2          CHILD 3
  * (print generator)  (lpr monitor)        (print command)  (error monitor)
  * |
@@ -1465,9 +1462,9 @@ usr1_handler (int sig)
  * | return
  * X
  *
- * 
+ *
  *                            ERROR CASE (job cancelled)
- * 
+ *
  * PARENT             CHILD 1              CHILD 2          CHILD 3
  * (print generator)  (lpr monitor)        (print command)  (error monitor)
  * |
@@ -1598,7 +1595,7 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
 		      if (opid < 0)
 			_exit(1);
 		      else if (opid == 0) /* Child 3 (monitors stderr) */
-			{ 
+			{
 			  stp_outfunc_t errfunc = stpui_get_errfunc();
 			  void *errdata = stpui_get_errdata();
 			  /* calls g_message on anything it sees */
@@ -1637,7 +1634,6 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
 		      else	/* Child 2 (printer command) */
 			{
 			  char *command;
-			  char *locale;
 			  if (stpui_plist_get_command_type(printer) ==
 			      COMMAND_TYPE_DEFAULT)
 			    {
@@ -1648,7 +1644,7 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
 			    }
 			  else
 			    command =
-			      (char *) stpui_plist_get_custom_command(printer);
+			      cast_safe(stpui_plist_get_custom_command(printer));
 			  (void) close(2);
 			  (void) close(1);
 			  dup2 (errfd[1], 2);
@@ -1658,7 +1654,7 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
 			  close (pipefd[1]);
 			  close(syncfd[1]);
 #ifdef HAVE_LOCALE_H
-			  locale = g_strdup(setlocale(LC_NUMERIC, NULL));
+			  setlocale(LC_NUMERIC, NULL);
 			  setlocale(LC_NUMERIC, "C");
 #endif
 			  execl("/bin/sh", "/bin/sh", "-c", command, NULL);
@@ -1795,8 +1791,9 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
       stp_set_errfunc(np->v, stpui_get_errfunc());
       stp_set_outdata(np->v, prn);
       stp_set_errdata(np->v, stpui_get_errdata());
+      stp_start_job(np->v, &(image->im));
       print_status = stp_print(np->v, &(image->im));
-
+      stp_end_job(np->v, &(image->im));
       /*
        * Note that we do not use popen() to create the output, therefore
        * we do not use pclose() to close it.  See bug 1013565.
@@ -1828,11 +1825,8 @@ stpui_print(const stpui_plist_t *printer, stpui_image_t *image)
 	}
       stpui_plist_destroy(np);
       g_free(np);
-      return 1;
+      return print_status;
     }
+
   return 0;
 }
-
-/*
- * End of "$Id: plist.c,v 1.18 2008/07/04 14:29:28 rlk Exp $".
- */
